@@ -44,74 +44,16 @@ struct fileops port_mio_ops = {
 	port_mio_hup
 };
 
-/*
- * open the port using one of the provided paths
- */
-static struct mio_hdl *
-port_mio_openlist(struct port *c, unsigned int mode)
-{
-	struct mio_hdl *hdl;
-	struct name *n;
-	int idx;
-
-	idx = 0;
-	n = c->path_list;
-	while (1) {
-		if (n == NULL)
-			break;
-		hdl = mio_open(n->str, mode, 1);
-		if (hdl != NULL) {
-			if (log_level >= 2) {
-				port_log(c);
-				log_puts(": using ");
-				log_puts(n->str);
-				log_puts("\n");
-			}
-			return hdl;
-		}
-		n = n->next;
-		idx++;
-	}
-	return NULL;
-}
-
 int
 port_mio_open(struct port *p)
 {
-	p->mio.hdl = port_mio_openlist(p, p->midi->mode);
+	p->mio.hdl = mio_open(p->path, p->midi->mode, 1);
 	if (p->mio.hdl == NULL)
 		return 0;
 	p->mio.file = file_new(&port_mio_ops, p, "port", mio_nfds(p->mio.hdl));
 	return 1;
 }
 
-/*
- * Open an alternate port. Upon success, close the old port
- * and continue using the new one.
- */
-int
-port_mio_reopen(struct port *p)
-{
-	struct mio_hdl *hdl;
-
-	hdl = port_mio_openlist(p, p->midi->mode);
-	if (hdl == NULL) {
-		if (log_level >= 1) {
-			port_log(p);
-			log_puts(": couldn't open an alternate port\n");
-		}
-		return 0;
-	}
-
-	/* close unused device */
-	file_del(p->mio.file);
-	mio_close(p->mio.hdl);
-
-	p->mio.hdl = hdl;
-	p->mio.file = file_new(&port_mio_ops, p, "port", mio_nfds(hdl));
-	return 1;
-}
-
 void
 port_mio_close(struct port *p)
 {
@@ -186,9 +128,8 @@ port_mio_hup(void *arg)
 {
 	struct port *p = arg;
 
-	if (!port_reopen(p)) {
-		midi_abort(p->midi);
-		if (p->state != PORT_CFG)
-			port_close(p);
-	}
+	port_migrate(p);
+	midi_abort(p->midi);
+	if (p->state != PORT_CFG)
+		port_close(p);
 }
