@@ -58,6 +58,13 @@ struct sock {
 	struct slot *slot;		/* audio device slot number */
 	struct midi *midi;		/* midi endpoint */
 	struct port *port;		/* midi port */
+	struct ctlslot *ctlslot;
+	struct amsg_ctl_desc *ctldesc;	/* temporary buffer */
+#define SOCK_CTLDESC	1		/* dump desc and send changes */
+#define SOCK_CTLVAL	2		/* send value changes */
+	unsigned int ctlops;		/* bitmap of above */
+	int ctlsyncpending;		/* CTLSYNC waiting to be transmitted */
+	unsigned int sesrefs;		/* 1 if socket belongs to a session */
 };
 
 struct sock *sock_new(int fd);
